/**
 * @file log.h
 * @brief 日志系统核心模块（支持多级日志、断言和异常处理）
 */

#ifndef __SMCC_LOG_H__
#define __SMCC_LOG_H__

#include "../std/rt_api_def.h"
#include "color.h"

/**
 * @brief 日志级别枚举
 * 
 * 定义日志系统的输出级别和组合标志位
 */
typedef enum log_level {
    LOG_LEVEL_NOTSET  = 0,      ///< 未设置级别（继承默认配置）
    LOG_LEVEL_DEBUG   = 1 << 0, ///< 调试信息（开发阶段详细信息）
    LOG_LEVEL_INFO    = 1 << 1, ///< 常规信息（系统运行状态）
    LOG_LEVEL_WARN    = 1 << 2, ///< 警告信息（潜在问题提示）
    LOG_LEVEL_ERROR   = 1 << 3, ///< 错误信息（可恢复的错误）
    LOG_LEVEL_FATAL   = 1 << 4, ///< 致命错误（导致程序终止的严重错误）
    LOG_LEVEL_TRACE   = 1 << 5, ///< 追踪（性能追踪或者栈帧追踪）
    LOG_LEVEL_ALL     = 0xFF,   ///< 全级别标志（组合所有日志级别）
} log_level_t;

/**
 * @brief 日志处理回调函数类型
 * @param level 日志级别
 * @param module 模块名称（可为NULL）
 * @param file 源文件名
 * @param line 代码行号
 * @param message 格式化后的日志消息
 * @todo 待实现模块名称，输入的模块名称，都将被忽略
 */
typedef void (*log_handler)(
    log_level_t level,
    const char* module,
    const char* file,
    int line,
    const char* message
);

#ifndef LOGGER_MAX_BUF_SIZE
#define LOGGER_MAX_BUF_SIZE 512  ///< 单条日志最大缓冲区尺寸
#endif

/**
 * @brief 日志器实例结构体
 * 
 * 每个日志器实例维护独立的配置和缓冲区
 */
typedef struct logger {
    const char* name;           ///< 日志器名称（用于模块区分）
    log_level_t level;          ///< 当前设置的日志级别
    log_handler handler;        ///< 日志处理回调函数
    char buf[LOGGER_MAX_BUF_SIZE]; ///< 格式化缓冲区
} logger_t;

/**
 * @brief 初始化日志实例 其余参数设置为默认值
 * @param[in] logger 日志器实例指针
 * @param[in] name 日志器名称（NULL表示获取默认日志器名称）
 */
void init_logger(logger_t* logger, const char* name);

// TODO log_set(); 暂未实现 日志注册

/**
 * @brief 获取或创建日志器实例
 * @param[in] name 日志器名称（NULL表示获取默认日志器）
 * @return 日志器实例指针
 * @warning 若没有找到相应日志器则会返回根日志器
 */
logger_t* log_get(const char* name);

/**
 * @brief 设置日志级别
 * @param[in] logger 目标日志器实例
 * @param[in] level 要设置的日志级别（可组合多个级别）
 */
void log_set_level(logger_t* logger, log_level_t level);

/**
 * @brief 设置自定义日志处理器
 * @param[in] logger 目标日志器实例
 * @param[in] handler 自定义处理函数（NULL恢复默认处理）
 */
void log_set_handler(logger_t* logger, log_handler handler);

#ifndef LOG_MAX_MAROC_BUF_SIZE
#define LOG_MAX_MAROC_BUF_SIZE LOGGER_MAX_BUF_SIZE  ///< 宏展开缓冲区尺寸
#endif

/**
 * @def _LOG
 * @brief 内部日志宏（供其他日志宏调用）
 * @param _module_ 模块实例（NULL表示使用默认日志器）
 * @param _level_ 日志级别
 * @param _msg_ 格式字符串
 * @param ... 可变参数列表
 */
#define _LOG(_module_, _level_, _msg_, ...) \
    do { \
        logger_t* _logger; \
        if (!_module_) { \
            _logger = log_get(NULL); \
        } \
        else _logger = _module_; \
        if (_logger && _logger->handler && (_logger->level & (_level_))) { \
            rt.snprintf(_logger->buf, sizeof(_logger->buf), (_msg_), ##__VA_ARGS__); \
            _logger->handler((_level_), _logger->name, __FILE__, __LINE__, _logger->buf); \
        } \
    } while(0)

/// @name 模块日志宏
/// @{
#define MLOG_NOTSET(module, ...) _LOG(module, LOG_LEVEL_NOTSET, __VA_ARGS__) ///< 未分类日志
#define MLOG_DEBUG( module, ...) _LOG(module, LOG_LEVEL_DEBUG,  __VA_ARGS__) ///< 调试日志（需启用DEBUG级别）
#define MLOG_INFO(  module, ...) _LOG(module, LOG_LEVEL_INFO,   __VA_ARGS__) ///< 信息日志（常规运行日志）
#define MLOG_WARN(  module, ...) _LOG(module, LOG_LEVEL_WARN,   __VA_ARGS__) ///< 警告日志（潜在问题）
#define MLOG_ERROR( module, ...) _LOG(module, LOG_LEVEL_ERROR,  __VA_ARGS__) ///< 错误日志（可恢复错误）
#define MLOG_FATAL( module, ...) _LOG(module, LOG_LEVEL_FATAL,  __VA_ARGS__) ///< 致命错误日志（程序终止前）
#define MLOG_TRACE( module, ...) _LOG(module, LOG_LEVEL_TRACE,  __VA_ARGS__) ///< 追踪日志（调用栈跟踪）
/// @}

/// @name 快捷日志宏
/// @{
#define LOG_NOTSET(...) _LOG(NULL, LOG_LEVEL_NOTSET, __VA_ARGS__) ///< 未分类日志
#define LOG_DEBUG(...)  _LOG(NULL, LOG_LEVEL_DEBUG,  __VA_ARGS__) ///< 调试日志（需启用DEBUG级别）
#define LOG_INFO(...)   _LOG(NULL, LOG_LEVEL_INFO,   __VA_ARGS__) ///< 信息日志（常规运行日志）
#define LOG_WARN(...)   _LOG(NULL, LOG_LEVEL_WARN,   __VA_ARGS__) ///< 警告日志（潜在问题）
#define LOG_ERROR(...)  _LOG(NULL, LOG_LEVEL_ERROR,  __VA_ARGS__) ///< 错误日志（可恢复错误）
#define LOG_FATAL(...)  _LOG(NULL, LOG_LEVEL_FATAL,  __VA_ARGS__) ///< 致命错误日志（程序终止前）
#define LOG_TRACE(...)  _LOG(NULL, LOG_LEVEL_TRACE,  __VA_ARGS__) ///< 追踪日志（调用栈跟踪）
/// @}

/**
 * @def _Assert
 * @brief 断言检查内部宏
 * @param cond 检查条件表达式
 * @param ... 错误信息参数（格式字符串+参数）
 */
#define _Assert(cond, ...) \
    do { \
        if (!(cond)) { \
            LOG_FATAL(__VA_ARGS__); \
        } \
    } while (0)

/// @name 断言工具宏
/// @{
#define AssertFmt(cond, format, ...) _Assert(cond, "Assertion Failure: " format, ## __VA_ARGS__) ///< 带格式的断言检查
#define PanicFmt(format, ...) _Assert(0, "Panic: " format, ## __VA_ARGS__) ///< 立即触发致命错误
#define Assert(cond) AssertFmt(cond, "cond is `" SMCC_STR(cond) "`") ///< 基础断言检查
#define Panic(...) PanicFmt(__VA_ARGS__) ///< 触发致命错误（带自定义消息）
#define TODO() PanicFmt("TODO please implement me") ///< 标记未实现代码（触发致命错误）
/// @}

#endif // __SMCC_LOG_H__
