#include "riscv32_mcode.h"

static const rv32_instr_t rv32_instrs[] = {
    #define X(ext, fmt, type, name, op) \
        [type] = { \
            .extention = ext, \
            .format = fmt, \
            .instr_type = type, \
            .instr_name = name, \
            .instr = op \
        },
    RV_INSTRUCTIONS
    #undef X
};

static inline int valid_reg(rv_reg_t reg) {
    // return reg >= REG_X0 && reg <= REG_X31;
    return !(reg & ~0x1F);
}

void init_rv32_mcode(mcode_rv32_t* mcode, int target_size) {
    vector_init(mcode->code);
    vector_init(mcode->rinstrs);
    mcode->target_size = target_size;
}

int decode_rv32_instr(u32_t instr, rv32_instr_t *out) {
    return 0;
}

int emit_rv32_instr(mcode_rv32_t* mcode, rv32_instr_t *instr, u32_t idx, void *target) {
    // TODO check out input data
    instr->extention =    rv32_instrs[instr->instr_type].extention;
    instr->format =       rv32_instrs[instr->instr_type].format;
    instr->instr_name =   rv32_instrs[instr->instr_type].instr_name;
    instr->instr =        rv32_instrs[instr->instr_type].instr;

    switch (instr->format) {
        case RV_R_TYPE:
            instr->instr |= (instr->rd << 7) | (instr->rs1 << 15) | (instr->rs2 << 20);
            break;
        case RV_I_TYPE:
            instr->instr |= (instr->rd << 7) | (instr->rs1 << 15) | (instr->imm << 20);
            break;
        case RV_S_TYPE:
            instr->instr |= (instr->rs1 << 15) | (instr->rs2 << 20) |
            ((instr->imm & 0x1F) << 7) | ((instr->imm & 0xFE0) << 20);
            break;
        case RV_B_TYPE:
            instr->instr |= (instr->rs1 << 15) | (instr->rs2 << 20) |
                (((instr->imm) >> 12) & 0x1)  << 31 |  /* imm[12:12] -> instr[31:31] */ \
                (((instr->imm) >> 5)  & 0x3F) << 25 |  /* imm[10:5]  -> instr[30:25] */ \
                (((instr->imm) >> 1)  & 0xF)  << 8  |  /* imm[4:1]   -> instr[11:8]  */ \
                (((instr->imm) >> 11) & 0x1)  << 7;    /* imm[11:11] -> instr[7:7]   */
            break;
        case RV_U_TYPE:
            instr->instr |= (instr->rd << 7) | (instr->imm & 0xFFFFF000);
            break;
        case RV_J_TYPE:
            instr->instr |= (instr->rd << 7) |
                (((instr->imm) >> 20) & 0x1)  << 31 |  /* imm[20:20] -> instr[31:31] */ \
                (((instr->imm) >> 1)  & 0x3FF)<< 21 |  /* imm[10:1]  -> instr[30:21] */ \
                (((instr->imm) >> 11) & 0x1)  << 20 |  /* imm[11:11] -> instr[20:20] */ \
                (((instr->imm) >> 12) & 0xFF) << 12;   /* imm[19:12] -> instr[19:12] */
            break;
        default:
            Panic("Invalid instruction format");
    }

    if (target != NULL) {
        Assert(idx == -1);
        void* ptr;
        if (mcode->target_size >= 0) {
            ptr = salloc_alloc(mcode->target_size);
            if (ptr == NULL) {
                LOG_FATAL("malloc failure");
            }
            rt_memcpy(ptr, target, mcode->target_size);
        } else {
            ptr = target;
        }
        rotated_instr_t rinstr = {
            .instr = *instr,
            .pos = mcode->code.size,
            .target = ptr,
        };
        vector_push(mcode->rinstrs, rinstr);
        // GOTO idx == -1
    }

    if (idx == -1) {
        // 32bit instr
        int idx = mcode->code.size;
        vector_push(mcode->code, 0);
        vector_push(mcode->code, 0);
        vector_push(mcode->code, 0);
        vector_push(mcode->code, 0);
        u32_t* pinstr = (u32_t*)(&vector_at(mcode->code, idx));
        *pinstr = instr->instr;
        return idx;
    } else if (idx >= 0 && idx < mcode->code.size) {
        *(u32_t*)&vector_at(mcode->code, idx) = instr->instr;
        return mcode->code.size - 1;
    } else {
        Panic("Invalid instruction index");
    }
    return -1;
}
